@@ -1,4 +1,6 @@
 # pylint: disable=c-extension-no-member
+# pylint: disable=too-many-lines
+from enum import Enum
 import json
 import logging
 import os
@@ -25,6 +27,7 @@ if platform.system() == "Windows":
     import win32security
     import pywinauto
     import win32gui
+    from comtypes import COMError
 
 
 def write_element_info_as_json(
@@ -56,6 +59,37 @@ class UnknownWindowsBackendError(Exception):
 
 
 SUPPORTED_BACKENDS = ["uia", "win32"]
+WINDOWS_LOCATOR_STRATEGIES = {
+    "name": "name",
+    "class_name": "class_name",
+    "class": "class_name",
+    "control_type": "control_type",
+    "type": "control_type",
+    "automation_id": "automation_id",
+    "id": "automation_id",
+    "partial name": "partial name",
+    "regexp": "regexp",
+}
+
+
+class Speed(Enum):
+    """Values for pywinauto Timings"""
+
+    DEFAULT = 1
+    SLOW = 2
+    FAST = 3
+
+
+def to_speed(value):
+    """Convert value to Speed enum."""
+    if isinstance(value, Speed):
+        return value
+
+    sanitized = str(value).lower().strip().replace(" ", "_")
+    try:
+        return Speed[sanitized]
+    except KeyError as err:
+        raise ValueError(f"Unknown speed: {value}") from err
 
 
 class Windows(OperatingSystem):
@@ -172,13 +206,8 @@ class Windows(OperatingSystem):
         self.windowtitle = None
         self.logger = logging.getLogger(__name__)
         self.clipboard = Clipboard()
-
-    def __del__(self):
-        try:
-            # TODO: Do this as RF listener instead of __del__?
-            self.clipboard.clear_clipboard()
-        except RuntimeError as err:
-            self.logger.debug("Failed to clear clipboard: %s", err)
+        self.elements = None
+        self.controls = None
 
     def set_windows_backend(self, backend: str) -> None:
         """Set Windows backend which is used to interact with Windows
@@ -333,11 +362,19 @@ class Windows(OperatingSystem):
         }
         return self._add_app_instance(app, dialog=False, params=params)
 
-    # TODO: How to manage app launched by open_file
-    def open_file(self, filename: str) -> bool:
+    def open_file(
+        self,
+        filename: str,
+        windowtitle: str = None,
+        wildcard: bool = False,
+        timeout: int = 10,
+    ) -> bool:
         """Open associated application when opening file
 
         :param filename: path to file
+        :param windowtitle: name of the window
+        :param wildcard: set True for inclusive window title search, default False
+        :param timeout: time to wait for dialog to appear
         :return: True if application is opened, False if not
 
         Example:
@@ -351,15 +388,21 @@ class Windows(OperatingSystem):
         if platform.system() == "Windows":
             # pylint: disable=no-member
             os.startfile(filename)
-            return True
         elif platform.system() == "Darwin":
             subprocess.call(["open", filename])
-            return True
         else:
             subprocess.call(["xdg-open", filename])
-            return True
-
-        return False
+        returnvalue = True
+        if windowtitle:
+            app_instance = self.open_dialog(
+                windowtitle, wildcard=wildcard, timeout=timeout
+            )
+            if app_instance > 0:
+                self._apps[app_instance]["executable"] = filename
+                self._apps[app_instance]["startkeyword"] = "Open File"
+            else:
+                returnvalue = False
+        return returnvalue
 
     def open_executable(
         self,
@@ -367,6 +410,7 @@ class Windows(OperatingSystem):
         windowtitle: str,
         backend: str = None,
         work_dir: str = None,
+        wildcard: bool = False,
     ) -> int:
         """Open Windows executable. Window title name is required
         to get handle on the application.
@@ -376,6 +420,7 @@ class Windows(OperatingSystem):
         :param backend: set Windows backend, default None means using
          library default value
         :param work_dir: path to working directory, default None
+        :param wildcard: set True for inclusive window title search, default False
         :return: application instance id
 
         Example:
@@ -383,25 +428,28 @@ class Windows(OperatingSystem):
         .. code-block:: robotframework
 
             ${app1}    Open Executable   calc.exe  Calculator
+            ${app2}    Open Executable   notepad.exe  Notepad   wildcard=True
 
         """
         self.logger.info("Opening executable: %s - window: %s", executable, windowtitle)
         if backend:
             self.set_windows_backend(backend)
-        params = {
-            "executable": executable,
-            "windowtitle": windowtitle,
-            "startkeyword": "Open Executable",
-        }
-        self.windowtitle = windowtitle
+
         app = pywinauto.Application(backend=self._backend).start(
             cmd_line=executable, work_dir=work_dir
         )
-
-        return self._add_app_instance(app, dialog=False, params=params)
+        app_instance = self.open_dialog(windowtitle, wildcard=wildcard)
+        self._apps[app_instance]["app"] = app
+        self._apps[app_instance]["executable"] = executable
+        self._apps[app_instance]["startkeyword"] = "Open Executable"
+        return app_instance
 
     def open_using_run_dialog(
-        self, executable: str, windowtitle: str, timeout: int = 10
+        self,
+        executable: str,
+        windowtitle: str,
+        timeout: int = 10,
+        wildcard: bool = False,
     ) -> int:
         """Open application using Windows run dialog.
         Window title name is required to get handle on the application.
@@ -409,6 +457,7 @@ class Windows(OperatingSystem):
         :param executable: name of the executable
         :param windowtitle: name of the window
         :param timeout: time to wait for dialog to appear
+        :param wildcard: set True for inclusive window title search, default False
         :return: application instance id
 
         Example:
@@ -416,6 +465,7 @@ class Windows(OperatingSystem):
         .. code-block:: robotframework
 
             ${app1}    Open Using Run Dialog  notepad  Untitled - Notepad
+            ${app2}    Open Using Run Dialog  notepad  Notepad    wildcard=True
 
         """
         self.send_keys("{VK_LWIN down}r{VK_LWIN up}")
@@ -423,14 +473,17 @@ class Windows(OperatingSystem):
 
         self.send_keys_to_input(executable, send_delay=0.2, enter_delay=0.5)
 
-        app_instance = self.open_dialog(windowtitle, timeout=timeout)
-        self._apps[app_instance]["windowtitle"] = windowtitle
+        app_instance = self.open_dialog(windowtitle, timeout=timeout, wildcard=wildcard)
         self._apps[app_instance]["executable"] = executable
         self._apps[app_instance]["startkeyword"] = "Open Using Run Dialog"
         return app_instance
 
     def open_from_search(
-        self, executable: str, windowtitle: str, timeout: int = 10
+        self,
+        executable: str,
+        windowtitle: str,
+        timeout: int = 10,
+        wildcard: bool = False,
     ) -> int:
         """Open application using Windows search dialog.
         Window title name is required to get handle on the application.
@@ -438,6 +491,7 @@ class Windows(OperatingSystem):
         :param executable: name of the executable
         :param windowtitle: name of the window
         :param timeout: time to wait for dialog to appear
+        :param wildcard: set True for inclusive window title search, default False
         :return: application instance id
 
         Example:
@@ -445,6 +499,7 @@ class Windows(OperatingSystem):
         .. code-block:: robotframework
 
             ${app1}    Open From Search  calculator  Calculator
+            ${app2}    Open From Search  notepad  Notepad  wildcard=True
 
         """
         self.logger.info("Run from start menu: %s", executable)
@@ -453,8 +508,7 @@ class Windows(OperatingSystem):
 
         self.send_keys_to_input(executable)
 
-        app_instance = self.open_dialog(windowtitle, timeout=timeout)
-        self._apps[app_instance]["windowtitle"] = windowtitle
+        app_instance = self.open_dialog(windowtitle, timeout=timeout, wildcard=wildcard)
         self._apps[app_instance]["executable"] = executable
         self._apps[app_instance]["startkeyword"] = "Open From Search"
         return app_instance
@@ -471,6 +525,7 @@ class Windows(OperatingSystem):
 
             ${txt}    Get Spaced String   My name is Bond
             # ${txt} = My{VK_SPACE}name{VK_SPACE}is{VK_SPACE}Bond
+            Send Keys To Input  ${txt}
 
         """
         return text.replace(" ", "{VK_SPACE}")
@@ -535,7 +590,7 @@ class Windows(OperatingSystem):
         self.dlg = pywinauto.Desktop(backend=self._backend)[windowtitle]
         self.dlg.minimize()
 
-    def restore_dialog(self, windowtitle: str = None) -> None:
+    def restore_dialog(self, windowtitle: str = None) -> None:  # noqa: C901
         """Restore window by its title
 
         :param windowtitle: name of the window, default `None` means that
@@ -553,21 +608,40 @@ class Windows(OperatingSystem):
             Restore Dialog    Untitled - Notepad
 
         """
-        windowtitle = (
-            windowtitle or self._apps[self._active_app_instance]["windowtitle"]
-        )
+        # TODO. Handle too compled method
         self.logger.info("Restore dialog: %s", windowtitle)
-        app = pywinauto.Application().connect(title_re=".*%s" % windowtitle)
+        if windowtitle is None and self._active_app_instance == -1:
+            raise ValueError(
+                "There are no applications opened by library or window title is empty"
+            )
+        app = None
+        handle = None
+        if windowtitle is None:
+            app = self._apps[self._active_app_instance]["app"]
+            handle = self._apps[self._active_app_instance]["handle"]
+            windowtitle = self._apps[self._active_app_instance]["windowtitle"]
+        else:
+            for app_id, app in self._apps.items():
+                if "windowtitle" in app.keys() and windowtitle == app["windowtitle"]:
+                    application_id = app_id
+                    app = self._apps[application_id]["app"]
+                    handle = self._apps[application_id]["handle"]
         try:
-            app.window().restore()
+            if app and handle > 0:
+                app.window(handle=handle).restore()
+            else:
+                wins = self.get_window_list()
+                for win in wins:
+                    if windowtitle == win["title"]:
+                        handle = win["handle"]
+                        break
+                if handle and handle > 0:
+                    self.connect_by_handle(handle, windowtitle=windowtitle)
+                    self.restore_dialog()
+                else:
+                    raise ValueError("Could not restore dialog: %s" % windowtitle)
         except pywinauto.findwindows.ElementAmbiguousError as e:
             self.logger.info("Could not restore dialog, %s", str(e))
-        finally:
-            if "handle" in self._apps[self._active_app_instance]:
-                app = pywinauto.Application().connect(
-                    handle=self._apps[self._active_app_instance]["handle"]
-                )
-                app.window().restore()
 
     def open_dialog(
         self,
@@ -575,12 +649,16 @@ class Windows(OperatingSystem):
         highlight: bool = False,
         timeout: int = 10,
         existing_app: bool = False,
+        wildcard: bool = False,
     ) -> Any:
         """Open window by its title.
 
         :param windowtitle: name of the window, defaults to active window if None
-        :param highlight: draw outline for window if True, defaults to False
+        :param highlight: draw outline for window if True, default False
         :param timeout: time to wait for dialog to appear
+        :param existing_app: set True if selecting window which library has already
+         accessed, default False
+        :param wildcard: set True for inclusive window title search, default False
 
         Example:
 
@@ -588,25 +666,31 @@ class Windows(OperatingSystem):
 
             Open Dialog       Untitled - Notepad
             Open Dialog       Untitled - Notepad   highlight=True   timeout=5
+            Open Dialog       Notepad   wildcard=True
 
         """
-        self.logger.info("Open dialog: '%s', '%s'", windowtitle, highlight)
-
-        if windowtitle:
-            self.windowtitle = windowtitle
+        self.logger.info("Open dialog: '%s'", windowtitle)
 
         app_instance = None
         end_time = time.time() + float(timeout)
         while time.time() < end_time and app_instance is None:
             for window in self.get_window_list():
-                if window["title"] == self.windowtitle:
+                if (not wildcard and windowtitle == window["title"]) or (
+                    wildcard and windowtitle in window["title"]
+                ):
+                    self.windowtitle = window["title"]
                     app_instance = self.connect_by_handle(
-                        window["handle"], existing_app=existing_app
+                        window["handle"],
+                        windowtitle=self.windowtitle,
+                        existing_app=existing_app,
                     )
+                    break
             time.sleep(0.1)
 
-        if self.dlg is None:
-            raise ValueError("No window with title '{}'".format(self.windowtitle))
+        if app_instance is None:
+            raise ValueError(
+                "No window with title '%s', wildcard: %s" % (windowtitle, wildcard)
+            )
 
         if highlight:
             self.dlg.draw_outline()
@@ -617,6 +701,7 @@ class Windows(OperatingSystem):
         """Connect to application by its pid
 
         :param app_pid: process id of the application
+        :param windowtitle: name of the window, defaults to active window if None
 
         Example:
 
@@ -637,11 +722,14 @@ class Windows(OperatingSystem):
         return None
 
     def connect_by_handle(
-        self, handle: str, windowtitle: str = None, existing_app: bool = False
+        self, handle: int, windowtitle: str = None, existing_app: bool = False
     ) -> Any:
         """Connect to application by its handle
 
         :param handle: handle of the application
+        :param windowtitle: name of the window, defaults to active window if None
+        :param existing_app: set True if selecting window which library has already
+         accessed, default False
 
         Example:
 
@@ -656,12 +744,18 @@ class Windows(OperatingSystem):
             handle=handle, visible_only=False
         )
         self.dlg = app.window(handle=handle)
-        self.dlg.restore()
+        self.dlg.set_focus()
         params = None
-        if not existing_app:
+        if existing_app:
+            for key in self._apps:
+                if self._apps[key]["handle"] == handle:
+                    app_instance = key
+                    break
+        else:
             if windowtitle is not None:
                 params = {"windowtitle": windowtitle}
             app_instance = self._add_app_instance(app=app, params=params, dialog=False)
+        self.refresh_window()
         return app_instance
 
     def close_all_applications(self) -> None:
@@ -688,6 +782,7 @@ class Windows(OperatingSystem):
         active application if `app_id` is None.
 
         :param app_id: application_id, defaults to None
+        :param send_keys: if ALT+F4 should be used to quit, default False
 
         Example:
 
@@ -701,16 +796,24 @@ class Windows(OperatingSystem):
         app = self.get_app(app_id)
         self.logger.info("Quit application: %s (%s)", app_id, app)
         if send_keys:
+            self.logger.info("Quit by F4 shortcut")
             self.switch_to_application(app_id)
             self.send_keys("%{F4}")
         else:
             if app["dispatched"]:
+                self.logger.info("Quit by app.Quit()")
                 app["app"].Quit()
             else:
-                if "process" in app and app["process"] > 0:
+                if "process_id" in app and app["process_id"] > 0:
                     # pylint: disable=E1101
-                    self.kill_process_by_pid(app["process"])
+                    pid = app["process_id"]
+                    if self.process_id_exists(pid):
+                        self.logger.info("Quit by killing process id")
+                        self.kill_process_by_pid(pid)
+                    else:
+                        self.logger.info("Process pid '%s' did not exist anymore", pid)
                 else:
+                    self.logger.info("Quit by app.kill()")
                     app["app"].kill()
         self._active_app_instance = -1
 
@@ -736,6 +839,7 @@ class Windows(OperatingSystem):
 
         :param locator: element locator
         :param keys:    list of keys to type
+        :param empty_field: if field should be emptied before typing, default False
 
         Example:
 
@@ -765,7 +869,7 @@ class Windows(OperatingSystem):
         .. code-block:: robotframework
 
             Open Executable  calc.exe  Calculator
-            Send Keys        2{+}3=
+            Send Keys        2{+}3{=}
 
         """
         pywinauto.keyboard.send_keys(keys)
@@ -814,6 +918,7 @@ class Windows(OperatingSystem):
         image: str = None,
         method: str = "locator",
         ctype: str = "click",
+        focus: str = "center",
         **kwargs,
     ) -> None:
         # pylint: disable=C0301
@@ -836,6 +941,8 @@ class Windows(OperatingSystem):
         :param image: image to click on desktop
         :param method: one of the available methods to mouse click, default "locator"
         :param ctype: type of mouse click
+        :param focus: default point for element click is 'center', can be set to 'topleft'
+         to click top left corner of the element
         :param kwargs: these keyword arguments can be used to pass arguments
          to underlying `Images` library to finetune image template matching,
          for example. `tolerance=0.5` would adjust image tolerance for the image
@@ -853,6 +960,7 @@ class Windows(OperatingSystem):
             FOR  ${element}  IN  @{elements}
                 Run Keyword If   ${element}[visible]   Mouse Click  ${element}
             END
+            Mouse Click  id:TrickyCheckbox  focus=topleft
 
         """  # noqa: E501
         self.logger.info("Mouse click: %s", locator)
@@ -869,7 +977,12 @@ class Windows(OperatingSystem):
                     raise ValueError(f"Could not find unique element for '{locator}'")
             if target_element is None:
                 raise ValueError("Could not find unique element to click")
-            x, y = self.get_element_center(target_element)
+            if focus == "topleft":
+                x, y, _, _ = self._get_element_coordinates(target_element)
+                x += 2
+                y += 2
+            else:
+                x, y = self.get_element_center(target_element)
             self.click_type(x + off_x, y + off_y, ctype)
         elif method == "coordinates":
             self.mouse_click_coords(x, y, ctype)
@@ -956,22 +1069,13 @@ class Windows(OperatingSystem):
             self.open_dialog(self.windowtitle)
         self.dlg.wait("exists enabled visible ready")
 
-        search_criteria, locator = self._determine_search_criteria(locator)
-        matching_elements, locators = self.find_element(locator, search_criteria)
-
-        locators = sorted(set(locators))
-        if locator in locators:
-            locators.remove(locator)
-        locators_string = "\n\t- ".join(locators)
+        matching_elements, _ = self.find_element(locator)
 
         if len(matching_elements) == 0:
             self.logger.info(
-                "Locator '%s' using search criteria '%s' not found in '%s'.\n"
-                "Maybe one of these would be better?\n%s\n",
+                "Locator '%s' not found in '%s'.\n",
                 locator,
-                search_criteria,
                 self.windowtitle,
-                locators_string,
             )
         elif len(matching_elements) == 1:
             element = matching_elements[0]
@@ -984,11 +1088,9 @@ class Windows(OperatingSystem):
             # TODO: return more valuable information about what should
             # be matching element ?
             self.logger.info(
-                "Locator '%s' matched multiple elements in '%s'. "
-                "Maybe one of these would be better?\n%s\n",
+                "Locator '%s' matched multiple elements in '%s'. ",
                 locator,
                 self.windowtitle,
-                locators_string,
             )
         return False
 
@@ -1135,12 +1237,13 @@ class Windows(OperatingSystem):
             @{elements}  Wait For Element  Results   timeout=10  interval=1.5
 
         """
+        self.refresh_window()
         end_time = time.time() + float(timeout)
         interval = max([0.5, interval])
         elements = None
         while time.time() < end_time:
             elements, _ = self.find_element(locator, search_criteria)
-            if len(elements) > 1:
+            if len(elements) > 0:
                 break
             if interval >= timeout:
                 self.logger.info(
@@ -1171,27 +1274,26 @@ class Windows(OperatingSystem):
             Log Many  ${elements[1]}     # list of all available locators
 
         """
-        search_locator = locator
-        if search_criteria is None:
-            search_criteria, search_locator = self._determine_search_criteria(locator)
-
-        controls, elements = self.get_window_elements()
-        self.logger.info(
-            "Find element: (locator: %s, criteria: %s)",
-            locator,
-            search_criteria,
-        )
+        match_type, search_locators = self._parse_locator(locator, search_criteria)
+        if self.elements is None:
+            controls, elements = self.get_window_elements()
+        else:
+            controls = self.controls
+            elements = self.elements
 
         matching_elements, locators = [], []
+
         for ctrl, element in zip(controls, elements):
-            if self.is_element_matching(element, search_locator, search_criteria):
+            match_results = {}
+            for criteria, search_locator in search_locators:
+                match_results[criteria] = self.is_element_matching(
+                    element, search_locator, criteria
+                )
+            if (match_type == "all" and all(match_results.values())) or (
+                match_type == "any" and any(match_results.values())
+            ):
                 element["control"] = ctrl
                 matching_elements.append(element)
-            if search_criteria == "any" and "name" in element:
-                locators.append(element["name"])
-            elif search_criteria and search_criteria in element:
-                locators.append(element[search_criteria])
-
         return matching_elements, locators
 
     def _determine_search_criteria(self, locator: str) -> Any:
@@ -1276,6 +1378,7 @@ class Windows(OperatingSystem):
         """Get dialog rectangle coordinates
 
         If `ctrl` is None then get coordinates from `dialog`
+
         :param ctrl: name of the window control object, defaults to None
         :return: coordinates: left, top, right, bottom
 
@@ -1377,16 +1480,14 @@ class Windows(OperatingSystem):
             @{elements}   Get Window Elements  screenshot=True  element_json=True  outline=True
 
         """  # noqa: E501
-        if self.dlg is None:
-            raise ValueError("No dialog open")
-
-        ctrls = [self.dlg]
-        if hasattr(self.dlg, "descendants"):
-            ctrls += self.dlg.descendants()
-
+        ctrls = self._get_all_window_controls()
         elements, controls = [], []
         for _, ctrl in enumerate(ctrls):
-            if not hasattr(ctrl, "element_info"):
+            try:
+                if not hasattr(ctrl, "element_info"):
+                    continue
+            except COMError as ce:
+                self.logger.info("Got COM error: %s", str(ce))
                 continue
 
             filename = clean_filename(
@@ -1414,32 +1515,43 @@ class Windows(OperatingSystem):
 
         return controls, elements
 
-    def _get_element_coordinates(self, rectangle: Any) -> Any:
+    def _get_all_window_controls(self):
+        if self.dlg is None:
+            raise ValueError("No dialog open")
+
+        ctrls = [self.dlg]
+        try:
+            if hasattr(self.dlg, "descendants"):
+                ctrls += self.dlg.descendants()
+        except COMError as ce:
+            self.logger.info("Got COM error: %s", str(ce))
+        return ctrls
+
+    def _get_element_coordinates(self, element: Any) -> Any:
         """Get element coordinates from pywinauto object.
 
         :param rectangle: item containing rectangle information
         :return: coordinates: left, top, right, bottom
         """
-        self.logger.debug(
-            "Get element coordinates from rectangle: %s of type %s",
-            rectangle,
-            type(rectangle),
-        )
         left = 0
         top = 0
         right = 0
         bottom = 0
-        if isinstance(rectangle, pywinauto.win32structures.RECT):
-            left = rectangle.left
-            top = rectangle.top
-            right = rectangle.right
-            bottom = rectangle.bottom
-        elif isinstance(rectangle, dict):
-            left = rectangle.left
-            top = rectangle.top
-            right = rectangle.right
-            bottom = rectangle.bottom
+        if isinstance(element, pywinauto.win32structures.RECT):
+            left = element.left
+            top = element.top
+            right = element.right
+            bottom = element.bottom
+        elif isinstance(element, dict) and "rectangle" not in element.keys():
+            left = element.left
+            top = element.top
+            right = element.right
+            bottom = element.bottom
         else:
+            if isinstance(element, dict) and "rectangle" in element.keys():
+                rectangle = element["rectangle"]
+            else:
+                rectangle = element
             left, top, right, bottom = map(
                 int,
                 re.match(
@@ -1547,17 +1659,25 @@ class Windows(OperatingSystem):
         element_attributes = [a for a in dir(element_info) if not a.startswith("_")]
 
         for attr in element_attributes:
-            attr_value = getattr(element_info, attr)
             try:
+                attr_value = getattr(element_info, attr)
                 element_dict[attr] = (
                     attr_value() if callable(attr_value) else str(attr_value)
                 )
             except TypeError:
                 pass
+            except COMError as ce:
+                self.logger.info("Got COM error: %s", str(ce))
 
         for attr in attributes_to_remove:
             element_dict.pop(attr, None)
 
+        element_dict["legacy"] = (
+            element.legacy_properties()
+            if hasattr(element, "legacy_properties")
+            else None
+        )
+        element_dict["object"] = element
         return element_dict
 
     def put_system_to_sleep(self) -> None:
@@ -1736,7 +1856,6 @@ class Windows(OperatingSystem):
                 self.restore_dialog(src["windowtitle"])
             for idx, selection in enumerate(selections):
                 self.logger.debug("Selecting item %d by mouse_click", idx)
-                self.logger.debug(selection)
                 # pywinauto.mouse.click(coords=(selection[0]+5, selection[1]+5))
                 self.mouse_click_coords(selection[0] + 5, selection[1] + 5)
 
@@ -1813,9 +1932,6 @@ class Windows(OperatingSystem):
         for w in windows:
             try:
                 left, top, right, bottom = self._get_element_coordinates(w.rectangle())
-                # Do not add windows into list if its rectangle area is zero
-                if left == right and bottom == top:
-                    continue
                 window_list.append(
                     {
                         "automation_id": w.automation_id(),
@@ -1831,3 +1947,57 @@ class Windows(OperatingSystem):
             except Exception as e:  # pylint: disable=broad-except
                 self.logger.debug(str(e))
         return window_list
+
+    def refresh_window(self):
+        """Get controls and elements for current windows.
+
+        Should be called always when window content changes on
+        Windows desktop.
+
+        :return: controls (list) and elements (list)
+        """
+        self.logger.debug("Refresh window")
+        controls, elements = self.get_window_elements()
+        self.elements = elements
+        self.controls = controls
+        return controls, elements
+
+    def _parse_locator(self, locator: str, search_criteria: str):
+        regex = rf"({':|'.join(WINDOWS_LOCATOR_STRATEGIES.keys())}:|or|and)('{{1}}(.+)'{{1}}|(\S+))?"  # noqa: E501
+        parts = re.finditer(regex, locator, re.IGNORECASE)
+
+        locators = []
+        match_type = "all"
+
+        for part in parts:
+            groups = part.groups()
+            if groups[0].lower() == "or":
+                match_type = "any"
+            elif groups[0].lower() == "and":
+                pass
+            else:
+                strategy, _ = groups[0].split(":")
+                value = groups[2] if groups[2] else groups[3]
+                locators.append([WINDOWS_LOCATOR_STRATEGIES[strategy], value])
+
+        # Add some strategies if there aren't other valid strategies
+        if not locators:
+            match_type = "any"
+            locators.append(["name", locator])
+            locators.append(["automation_id", locator])
+            locators.append([search_criteria, locator])
+        return match_type, locators
+
+    def set_automation_speed(self, speed: Speed = Speed.DEFAULT):
+        """Set global automation timings
+
+        :param speed: possible values 'default', 'fast' or 'slow'
+        """
+        speed = to_speed(speed)
+
+        if speed == Speed.DEFAULT:
+            pywinauto.timings.Timings.defaults()
+        elif speed == Speed.SLOW:
+            pywinauto.timings.Timings.slow()
+        elif speed == Speed.FAST:
+            pywinauto.timings.Timings.fast()
